#! env python

# Generate fake BU and traffic from MNs
# Guillaume Valadon <guedou@hongo.wide.ad.jp>

# Changelog:
# 01/02/2006: new cli args
# 19/01/2006: cli args and ping during BU's lifetime
# xx/12/2006: intial code

from scapy6 import *
conf.verb = 0
conf.iface = 'eth0'

import math, time, getopt, sys

# HA in K2
NUM=[1,2,3,4]
HOMEADDRESS='2001:200:0:8440::%s'
hal=['2001:200:0:8440::1000']
BU_TIME=2

TARGET='www.netbsd.org'

## from UTscapy.py
try:
    opts = getopt.getopt(sys.argv[1:], "a:m:t:l:h")
    for opt,optarg in opts[0]:
	if opt == "-h":
	    print >>sys.stderr, 'Usage: %s [ -a ha_address]  [ -m mn_addresses_range ] [ -t target ] [ -l bu_lifetime] [ -h ]' % sys.argv[0]
	    raise SystemExit
	elif opt == "-t":
	    TARGET = optarg
	elif opt == "-l":
	    BU_TIME = int(optarg)
	elif opt == "-a":
		HOMEADDRESS = optarg+'%s'
	elif opt == "-m":
	    NUM = optarg.split(':')
	    NUM = [ int(c) for c in NUM ]
	    if len(NUM) > 1:
		NUM = range(NUM[0], NUM[1]+1)
except getopt.GetoptError,msg:
    print >>sys.stderr,"ERROR:",msg
    raise SystemExit

def advertip6(ip6, src):
    x = socket.inet_pton(socket.AF_INET6, ip6)
    
    iff,a,nh = conf.route6.route_iface(ip6, conf.iface)
    if nh != '::':
	ip6 = nh

    nsma = in6_getnsma(socket.inet_pton(socket.AF_INET6, ip6))
    d = socket.inet_ntop(socket.AF_INET6, nsma)
    dm = in6_getnsmac(nsma)
    p = Ether(dst=dm)/IPv6(dst=d, src=src, hlim=255)
    p /= ICMPv6ND_NS(tgt=ip6)
    p /= ICMPv6NDOptSrcLLAddr(lladdr=get_if_hwaddr(iff))

    res = srp1(p,
	      type=ETH_P_IPV6,
	      iface=iff,
	      timeout=2,
	      verbose=0)
    if res is not None:
	mac = res.lladdr
	return mac
    return None

# prepare COAs    

COA = socket.inet_ntop(socket.AF_INET6, get_if_raw_addr6(conf.iface))

coa = {}
for i in NUM:
    coa[i] = COA[:-1]+str(i)

# send BUs (implicit mode)

for i in NUM:
    OK = 0
    bu = IPv6()/IPv6OptionHeaderHomeAddress(ha=HOMEADDRESS%i)/pad([IPv6MobilityHeader_BU(time=BU_TIME),AlternateCareofAddress(acoa=coa[i])])
    for ha in hal: # useless ?!?
	bu.dst = ha
	bu.src = coa[i]
        advertip6(bu.dst, bu.src)
	r = sr1(bu, timeout=5)
	if isinstance(r, IPv6) and IPv6MobilityHeader_BACK in r and r[IPv6MobilityHeader_BACK].status == 0:
	    OK = 1
	    break

    if not OK:
	if isinstance(r, IPv6) and IPv6MobilityHeader_BACK in r:
	    print 'BU rejected %s' % r.status
	sys.exit()
    else:
	print 'BACK received - COA %s' %  r.dst
	START = math.ceil(time.time()) # last BACK received !

# send ICMP packets from MNs using the tunnel until the end of the BU's lifetime

wait = BU_TIME*4-math.ceil((time.time()-START))
while wait > 0:
    for i in NUM:
	pkt = IPv6(dst=TARGET, src=HOMEADDRESS%i)/ICMPv6EchoRequest()
	r = sr1(IPv6(dst=ha, src=coa[i])/pkt, timeout=2)
	if isinstance(r, IPv6) and ICMPv6EchoReply in r:
	    print 'The MN (%s) received a reply from %s' % (HOMEADDRESS%i, r.payload.src)
	else:    
	    print 'The MN (%s) did not receive a reply' % HOMEADDRESS%i
	wait = BU_TIME*4-math.ceil((time.time()-START))
