#! /usr/bin/env python

# ChangeLog:
#
#  - 2005/06/21 guedou
#   Split detection and healing
#
# - 2005/06/20 guedou
#   Code cleanup
#   One node movement (too fast)


from graph import *

import random
import sys
import cPickle


MATRIX = 'matrix.dump'
EBTABLES = 'ebtables.sh'
HEAL_SPLIT = 0


def init(filename):
    f = open(filename, 'r')
    m = {}
    n = []
    while 1:
	lines = f.readline()
	if lines == '':
	    break
	l = lines[:-1].split(' ')
	n.append(l[0])       # IP
	m[l[0]] = []         
	m[l[0]].append(l[1]) # interface
	m[l[0]].append(l[2]) # MAC @
    f.close()
    return m,n


def initmatrix(l):
    m = []
    for i in xrange(0, len(l)):
	m.append(map(lambda x: 0, l))
	m[i][i] = -1
    return m


def buildgraph(m):
    for i in xrange(0, len(m)):
	def f(x): return x[1] == 0
	t = map(lambda x: x[0], filter(f, list(enumerate(m[i]))))
	if not t == []:	
	    n = t[random.randint(0, len(t) - 1)]
	    m[i][n] = 1
	    m[n][i] = 2
    return m


def ebfilters(dst, src, command='A'):  
    FILTER = 'ebtables -%s FORWARD -o %s -s %s -j ACCEPT #%s -> %s\n'

    f = open(EBTABLES, 'a')
    m = mac[dst][1]
    int = mac[src][0]
    f.write(FILTER % (command, int, m, dst, src))

    m = mac[src][1]
    int = mac[dst][0]
    f.write(FILTER % (command, int, m, src, dst))
    f.write('\n')
    f.close()

def depthsearch(m):
    f = []
    t = [0]

    def dsrec(f, t):
        while not t == []:
	    n = t.pop()
	    def p(x): return x[1] > 0
	    l = map(lambda x: x[0], filter(p, list(enumerate(m[n]))))
	    try:
		f.index(n)
	    except ValueError, err:		
		f.append(n)
		for i in l:
		    try:
			t.index(i)
		    except ValueError, err:		
			t.append(i)
	return f,t    

    f, t = dsrec(f, t)	    
    return f
  
mac, nodes = init('mac.txt')

new = (None, None)
old = (None, None)

try:
    matrix = cPickle.load(open(MATRIX, 'r'))
    print ' o Loading links matrix from file'
    LOADED = 1
except IOError, err:
    matrix = initmatrix(nodes)
    matrix = buildgraph(matrix)
    print ' o Generating new links matrix'
    LOADED = 0

if LOADED:
    r = random.randint(0, len(matrix) - 1)
    t = [] # Actual links
    n = [] # Possible new links
    for x,a in enumerate(matrix[r]): 
	if a in [1, 2]:
	    t.append(x)             
	if a == 0:
	    n.append(x)

    if not t == [] and not n == []:	    
	print ' o Moving node %s' % nodes[r]

    if not n == []:
	rn = n[random.randint(0, len(n) - 1)]
	print '   - Adding: %s -> %s' % (nodes[r], nodes[rn])
	matrix[r][rn] = 1
	matrix[rn][r] = 2
	new = r,rn
	ebfilters(nodes[r], nodes[rn])

    if not t == [] and len(t) > 1:
	rn = t[random.randint(0, len(t) - 1)]
	print '   - Suppressing: %s -> %s' % (nodes[r], nodes[rn])
	matrix[r][rn] = matrix[rn][r] = 0
	old = (r, rn)
	ebfilters(nodes[r], nodes[rn], command='D')

l = depthsearch(matrix)
if len(l) != len(matrix):
    print ' o Split detected: %s can\'t reach %i nodes' % (nodes[0], len(matrix) - len(l))

    if HEAL_SPLIT: 
	def listdiff(a, b):
	    if a == []:      return [] 
	    if b == []:      return a
	    if a[0] != b[0]: return a[0:1] + listdiff(a[1:], b)
	    else:            return listdiff(a[1:], b[1:])

	t = range(0, len(matrix))
	far = depthsearch(matrix)
	far.sort()
	far = listdiff(t, far)
	while len(l) != len(matrix):
	    po = far.pop()
	    matrix[0][po] = 1
	    matrix[po][0] = 2
	    print '   o New link: %s -> %s' % (nodes[0], nodes[po])
	    ebfilters(nodes[0], nodes[po])
	    l = depthsearch(matrix)

graph = dot('graph.dot')

for i in xrange(0, len(matrix)):
    for j in xrange(0, len(matrix[i])):
        ALONE = 1
	if matrix[i][j] == 1:
	    ALONE = 0
	    if (i, j) == new:
	      c = 'blue'
	      lab = 'ADDED'
	    else:
	      c = 'black'
	      lab = ''
	    graph.addlink(nodes[i], nodes[j], color=c, label=lab)  
	    if not LOADED:
                ebfilters(nodes[i], nodes[j])
    if ALONE:
	graph.addlink(nodes[i], nodes[0], color='white')

if (None, None) != old:
    graph.addlink(nodes[old[0]], nodes[old[1]], color='red', label='DELETED')  

graph.save()

print ' o Dumping links matrix to file'
cPickle.dump(matrix, open(MATRIX, 'w'))
